/**
 * Copyright (c) 2020 Obeo
 * This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License 2.0
 * which accompanies this distribution, and is available at
 * https://www.eclipse.org/legal/epl-2.0/
 * 
 * SPDX-License-Identifier: EPL-2.0
 * 
 * Contributors:
 *  Obeo - initial API and implementation
 */
package fr.obeo.dsl.tuto.mindstorms.layout;

import java.util.EnumSet;
import org.eclipse.elk.core.data.ILayoutMetaDataProvider;
import org.eclipse.elk.core.data.LayoutOptionData;
import org.eclipse.elk.graph.properties.IProperty;
import org.eclipse.elk.graph.properties.Property;

@SuppressWarnings("all")
public class MindstormsMetadataProvider implements ILayoutMetaDataProvider {
  /**
   * Default value for {@link #SPACING_LINE}.
   */
  private static final double SPACING_LINE_DEFAULT = 30;
  
  /**
   * Lower bound value for {@link #SPACING_LINE}.
   */
  private static final Comparable<? super Double> SPACING_LINE_LOWER_BOUND = Double.valueOf(0.0);
  
  /**
   * The vertical spacing to be preserved between each line, ie between the lowest node of the upper line and the highest node of the lower line.
   * Note that 'spacing.nodeNode' is used for the spacing between nodes within the line itself.
   */
  public static final IProperty<Double> SPACING_LINE = new Property<Double>(
            "fr.obeo.dsl.tuto.mindstorms.layout.spacing.line",
            SPACING_LINE_DEFAULT,
            SPACING_LINE_LOWER_BOUND,
            null);
  
  /**
   * Default value for {@link #NODE_PLACEMENT_NB_NODES_PER_LINE}.
   */
  private static final double NODE_PLACEMENT_NB_NODES_PER_LINE_DEFAULT = 4;
  
  /**
   * Lower bound value for {@link #NODE_PLACEMENT_NB_NODES_PER_LINE}.
   */
  private static final Comparable<? super Double> NODE_PLACEMENT_NB_NODES_PER_LINE_LOWER_BOUND = Double.valueOf(1.0);
  
  /**
   * The number of nodes to display per line.
   */
  public static final IProperty<Double> NODE_PLACEMENT_NB_NODES_PER_LINE = new Property<Double>(
            "fr.obeo.dsl.tuto.mindstorms.layout.nodePlacement.nbNodesPerLine",
            NODE_PLACEMENT_NB_NODES_PER_LINE_DEFAULT,
            NODE_PLACEMENT_NB_NODES_PER_LINE_LOWER_BOUND,
            null);
  
  public void apply(final org.eclipse.elk.core.data.ILayoutMetaDataProvider.Registry registry) {
    registry.register(new LayoutOptionData.Builder()
        .id("fr.obeo.dsl.tuto.mindstorms.layout.spacing.line")
        .group("spacing")
        .name("Line Spacing")
        .description("The vertical spacing to be preserved between each line, ie between the lowest node of the upper line and the highest node of the lower line. Note that \'spacing.nodeNode\' is used for the spacing between nodes within the line itself.")
        .defaultValue(SPACING_LINE_DEFAULT)
        .lowerBound(SPACING_LINE_LOWER_BOUND)
        .type(LayoutOptionData.Type.DOUBLE)
        .optionClass(Double.class)
        .targets(EnumSet.of(LayoutOptionData.Target.PARENTS))
        .visibility(LayoutOptionData.Visibility.VISIBLE)
        .create()
    );
    registry.register(new LayoutOptionData.Builder()
        .id("fr.obeo.dsl.tuto.mindstorms.layout.nodePlacement.nbNodesPerLine")
        .group("nodePlacement")
        .name("Nb Nodes Per Line")
        .description("The number of nodes to display per line.")
        .defaultValue(NODE_PLACEMENT_NB_NODES_PER_LINE_DEFAULT)
        .lowerBound(NODE_PLACEMENT_NB_NODES_PER_LINE_LOWER_BOUND)
        .type(LayoutOptionData.Type.DOUBLE)
        .optionClass(Double.class)
        .targets(EnumSet.of(LayoutOptionData.Target.PARENTS))
        .visibility(LayoutOptionData.Visibility.VISIBLE)
        .create()
    );
    new fr.obeo.dsl.tuto.mindstorms.layout.options.MindstormsOptions().apply(registry);
  }
}
